/*!\file: Sing.h
 * \brief prototypes for Sing element
 */ 

#ifndef _SING_H_
#define  _SING_H_

#include "./Element.h"
#include "./Node.h"
#include "./Matice.h"
#include "./Matpar.h"
#include "./ParameterInputs.h"

class Sing: public Element{

	private: 

		/*id:*/
		int id;

		/*node: */
		int   node_id; //node id
		Node* node; //node pointer
		int   node_offset; //node offset in nodes dataset

		/*material: */
		int   mid;
		Matice* matice; 
		int   matice_offset;
		
		int mparid;
		Matpar* matpar; 
		int   matpar_offset;
	
		double h;
		double k;

	public:

		Sing();
		Sing(int sing_id, int sing_mid, int sing_mparid, int sing_g, double sing_h, double sing_k);
		~Sing();

		void  Echo();
		void  DeepEcho();
		void  Marshall(char** pmarshalled_dataset);
		int   MarshallSize();
		char* GetName();
		void  Demarshall(char** pmarshalled_dataset);
		int   Enum();
		int   GetId(); 
		int   MyRank();
		void  Configure(void* loads,void* nodes,void* materials);
		void  CreateKMatrix(Mat Kgg,void* inputs,int analysis_type,int sub_analysis_type);
		void  CreatePVector(Vec pg, void* inputs, int analysis_type,int sub_analysis_type);
		void  UpdateFromInputs(void* inputs);
		void  GetDofList(int* doflist,int* pnumberofdofs);
		void  GetDofList1(int* doflist);
		void  CreateKMatrixDiagnosticHutter(Mat Kgg,void* inputs,int analysis_type,int sub_analysis_type);
		void  GetParameterValue(double* pp, double* plist, double* gauss_l1l2l3);
		void  CreatePVectorDiagnosticHutter(Vec pg,void* inputs,int analysis_type,int sub_analysis_type);
		void* GetMatPar();

		Object* copy();
		void  NodeConfiguration(int sing_node_id,Node* sing_node,int sing_node_offset);
		void  MaticeConfiguration(Matice* matice,int matice_offset);
		void  MatparConfiguration(Matpar* matpar,int matpar_offset);
		void  ComputePressure(Vec p_g);
		void  GetNodes(void** vpnodes);
		
		/*not implemented: */
		int   GetShelf();
		int   GetOnBed();
		void  GetBedList(double*);
		void  GetThicknessList(double* thickness_list);
		void  Du(_p_Vec*,void*,int,int);
		void  Gradj(_p_Vec*, void*, int, int,char*);
		void  GradjDrag(_p_Vec*, void*, int,int);
		void  GradjB(_p_Vec*, void*, int,int);
		double Misfit(void*,int,int);
		double MassFlux(double* segment,double* vx,double* vy,double* vz);



};
#endif //ifndef _SING_H_

